--
-- Custom overviews
--

CREATE TABLE custom_overview_tbl (
    id SERIAL PRIMARY KEY NOT NULL,
    bearbeiter_id INTEGER NOT NULL,
    name VARCHAR(128) NOT NULL,
    overview TEXT NOT NULL,

    -- Use ON DELETE CASCADE for the foreign key constraint on
    -- bearbeiter_id so that we do not have to explicitly delete a
    -- user's overviews when the user account is deleted. The overviews
    -- are not important enough to warrant explicit confirmation from
    -- the admin before they're deleted when the user account is
    -- deleted.
    FOREIGN KEY (bearbeiter_id) REFERENCES ka_benutzer_tbl(id) ON DELETE CASCADE
);


CREATE OR REPLACE VIEW custom_overview_tbl_view AS
SELECT custom_overview_tbl.id,
       custom_overview_tbl.bearbeiter_id,
       custom_overview_tbl.name,
       custom_overview_tbl.overview
FROM custom_overview_tbl
WHERE custom_overview_tbl.bearbeiter_id = getuserid();

CREATE OR REPLACE RULE custom_overview_tbl_view_update_rule
    AS ON UPDATE TO custom_overview_tbl_view
       DO INSTEAD
	  UPDATE custom_overview_tbl
	     SET name = new.name,
		 overview = new.overview
	   WHERE custom_overview_tbl.id = new.id;

GRANT SELECT ON custom_overview_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON custom_overview_tbl_view TO GROUP :cm_group;
GRANT SELECT ON custom_overview_tbl_view TO GROUP :admin_group;
GRANT UPDATE ON custom_overview_tbl_view TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION delete_custom_overview_ds(tid integer) RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
BEGIN
    DELETE FROM custom_overview_tbl
    WHERE id = tid AND getuserid() = bearbeiter_id;
END;
$$ ;

REVOKE ALL    ON FUNCTION delete_custom_overview_ds(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_custom_overview_ds(integer) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION delete_custom_overview_ds(integer) TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION create_custom_overview_ds(new_name VARCHAR(128), new_overview TEXT) RETURNS INTEGER
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
AS $$
DECLARE
     rid integer;
     bid integer;
BEGIN
     bid = getuserid();
     INSERT INTO custom_overview_tbl(bearbeiter_id, name, overview)
	  VALUES (bid, new_name, new_overview)
     RETURNING id INTO rid;
     RETURN rid;
END;
$$;

REVOKE ALL    ON FUNCTION create_custom_overview_ds(VARCHAR(128), TEXT) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_custom_overview_ds(VARCHAR(128), TEXT) TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION create_custom_overview_ds(VARCHAR(128), TEXT) TO GROUP :admin_group;
