--
-- Add fields to control check of copied Strukturdaten
-- mpuls/issue7522
--

--
-- The introduction of field entstand_aus_kopie affects the generation of 
-- the SQL schema: New key-value table for 'Keine Angabe'/'Ja'/'Nein'
-- fields is entstand_aus_kopie_tbl.
--
-- All foreign key contraints currently referencing ha_sprachkenntnis_arabisch_tbl
-- have to be redefined.

CREATE TABLE entstand_aus_kopie_tbl (
    id SERIAL PRIMARY KEY NOT NULL,
    value VARCHAR(13) NOT NULL
);

INSERT INTO entstand_aus_kopie_tbl (id, value) VALUES (-1, 'Keine Angabe');
INSERT INTO entstand_aus_kopie_tbl (id, value) VALUES (0, 'Nein');
INSERT INTO entstand_aus_kopie_tbl (id, value) VALUES (1, 'Ja');

GRANT SELECT ON entstand_aus_kopie_tbl TO GROUP :cm_group;
GRANT SELECT ON entstand_aus_kopie_tbl TO GROUP :admin_group;

--
-- master_tbl
--

-- new fields
ALTER TABLE master_tbl ADD COLUMN entstand_aus_kopie INTEGER DEFAULT -1;
ALTER TABLE master_tbl
  ADD FOREIGN KEY (entstand_aus_kopie)
      REFERENCES entstand_aus_kopie_tbl (id);

ALTER TABLE master_tbl ADD COLUMN kopie_geprueft_p_0 INTEGER DEFAULT -1;
ALTER TABLE master_tbl
  ADD FOREIGN KEY (kopie_geprueft_p_0)
      REFERENCES entstand_aus_kopie_tbl (id);

ALTER TABLE master_tbl ADD COLUMN kopie_geprueft_p_1 INTEGER DEFAULT -1;
ALTER TABLE master_tbl
  ADD FOREIGN KEY (kopie_geprueft_p_1)
      REFERENCES entstand_aus_kopie_tbl (id);

ALTER TABLE master_tbl ADD COLUMN kopie_geprueft_p_9 INTEGER DEFAULT -1;
ALTER TABLE master_tbl
  ADD FOREIGN KEY (kopie_geprueft_p_9)
      REFERENCES entstand_aus_kopie_tbl (id);

ALTER TABLE master_tbl ADD COLUMN kopie_geprueft_p_6 INTEGER DEFAULT -1;
ALTER TABLE master_tbl
  ADD FOREIGN KEY (kopie_geprueft_p_6)
      REFERENCES entstand_aus_kopie_tbl (id);

ALTER TABLE master_tbl ADD COLUMN kopie_geprueft_p_4 INTEGER DEFAULT -1;
ALTER TABLE master_tbl
  ADD FOREIGN KEY (kopie_geprueft_p_4)
      REFERENCES entstand_aus_kopie_tbl (id);

-- alter foreign key constraints
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_ha_sprachkenntnis_arabisch_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_ha_sprachkenntnis_englisch_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_ha_sprachkenntnis_franzoesisch_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_ha_sprachkenntnis_polnisch_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_ha_sprachkenntnis_russisch_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_ha_sprachkenntnis_spanisch_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_ha_sprachkenntnis_tuerkisch_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_ha_sprachkenntnis_sonstige_fkey RESTRICT;

ALTER TABLE master_tbl ADD FOREIGN KEY (ha_sprachkenntnis_arabisch) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (ha_sprachkenntnis_englisch) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (ha_sprachkenntnis_franzoesisch) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (ha_sprachkenntnis_polnisch) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (ha_sprachkenntnis_russisch) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (ha_sprachkenntnis_spanisch) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (ha_sprachkenntnis_tuerkisch) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (ha_sprachkenntnis_sonstige) REFERENCES entstand_aus_kopie_tbl (id);

ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_weiter_akt_art_interkulturell_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_weiter_akt_art_aufsuchende_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_weiter_akt_art_eltern_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_weiter_akt_art_freiwilligerdienst_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_weiter_akt_art_sonstiges_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_weiter_akt_art_keine_fkey RESTRICT;
ALTER TABLE master_tbl DROP CONSTRAINT master_tbl_eingabe_vollstaendig_fkey RESTRICT;

ALTER TABLE master_tbl ADD FOREIGN KEY (weiter_akt_art_interkulturell) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (weiter_akt_art_aufsuchende) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (weiter_akt_art_eltern) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (weiter_akt_art_freiwilligerdienst) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (weiter_akt_art_sonstiges) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (weiter_akt_art_keine) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE master_tbl ADD FOREIGN KEY (eingabe_vollstaendig) REFERENCES entstand_aus_kopie_tbl (id);


-- rg_netzwerk_tbl

ALTER TABLE rg_netzwerk_tbl ADD COLUMN kopie_geprueft_p_5 INTEGER DEFAULT -1;
ALTER TABLE rg_netzwerk_tbl
  ADD FOREIGN KEY (kopie_geprueft_p_5)
      REFERENCES entstand_aus_kopie_tbl (id);


-- rg_koop_bewertungen_tbl

ALTER TABLE rg_koop_bewertungen_tbl ADD COLUMN kopie_geprueft_p_11 INTEGER DEFAULT -1;
ALTER TABLE rg_koop_bewertungen_tbl
  ADD FOREIGN KEY (kopie_geprueft_p_11)
      REFERENCES entstand_aus_kopie_tbl (id);


-- rg_kjp_gruppenangebote_tbl

ALTER TABLE rg_kjp_gruppenangebote_tbl ADD COLUMN kopie_geprueft_p_2 INTEGER DEFAULT -1;
ALTER TABLE rg_kjp_gruppenangebote_tbl
  ADD FOREIGN KEY (kopie_geprueft_p_2)
      REFERENCES entstand_aus_kopie_tbl (id);

ALTER TABLE rg_kjp_gruppenangebote_tbl DROP CONSTRAINT rg_kjp_gruppenangebote_tbl_kjp_alter1_fkey RESTRICT;
ALTER TABLE rg_kjp_gruppenangebote_tbl DROP CONSTRAINT rg_kjp_gruppenangebote_tbl_kjp_alter2_fkey RESTRICT;
ALTER TABLE rg_kjp_gruppenangebote_tbl DROP CONSTRAINT rg_kjp_gruppenangebote_tbl_kjp_alter3_fkey RESTRICT;
ALTER TABLE rg_kjp_gruppenangebote_tbl ADD FOREIGN KEY (kjp_alter1) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE rg_kjp_gruppenangebote_tbl ADD FOREIGN KEY (kjp_alter2) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE rg_kjp_gruppenangebote_tbl ADD FOREIGN KEY (kjp_alter3) REFERENCES entstand_aus_kopie_tbl (id);



-- rg_nichtkjp_gruppenangebote_tbl

ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD COLUMN kopie_geprueft_p_3 INTEGER DEFAULT -1;
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl
  ADD FOREIGN KEY (kopie_geprueft_p_3)
      REFERENCES entstand_aus_kopie_tbl (id);

ALTER TABLE rg_nichtkjp_gruppenangebote_tbl DROP CONSTRAINT rg_nichtkjp_gruppenangebote_tbl_nichtkjp_alter1_fkey RESTRICT;
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl DROP CONSTRAINT rg_nichtkjp_gruppenangebote_tbl_nichtkjp_alter2_fkey RESTRICT;
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl DROP CONSTRAINT rg_nichtkjp_gruppenangebote_tbl_nichtkjp_alter3_fkey RESTRICT;
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD FOREIGN KEY (nichtkjp_alter1) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD FOREIGN KEY (nichtkjp_alter2) REFERENCES entstand_aus_kopie_tbl (id);
ALTER TABLE rg_nichtkjp_gruppenangebote_tbl ADD FOREIGN KEY (nichtkjp_alter3) REFERENCES entstand_aus_kopie_tbl (id);

DROP TABLE ha_sprachkenntnis_arabisch_tbl CASCADE;
