-- Issue 3858: ka_logbuch_tbl.notiz escaped
-- Fuer den Trigger benoetigen wir die notiz_from_plaintext Function.
CREATE OR REPLACE FUNCTION notiz_from_plaintext(notiz text) RETURNS text
AS $notiz_from_plaintext$
BEGIN
    RETURN  '<p>' || replace(replace(replace(replace(
                                notiz, '&', '&amp;'),
                                       '<', '&lt;'),
                                       '>', '&gt;'),
                                       E'\n', '<br>') || '</p>';
END;
$notiz_from_plaintext$ LANGUAGE plpgsql;


--
-- Update master_ds_editor_changed
--
-- Since it's a trigger functions, drop the trigger first and re-create
-- it afterwards.
DROP TRIGGER master_ds_editor_changed_trigger ON master_tbl;

CREATE OR REPLACE FUNCTION master_ds_editor_changed() RETURNS trigger AS $master_ds_editor_changed$
DECLARE
    lid      integer;
    old_user RECORD;
    new_user RECORD;
BEGIN
    IF OLD.bearbeiter_id <> NEW.bearbeiter_id THEN

        SELECT INTO old_user * FROM ka_benutzer_tbl WHERE id = OLD.bearbeiter_id;
        SELECT INTO new_user * FROM ka_benutzer_tbl WHERE id = NEW.bearbeiter_id;

        SELECT INTO lid create_logbuch_ds(NEW.id, NULL);

        UPDATE ka_logbuch_tbl_view
            SET art = 0, 
            notiz = notiz_from_plaintext('Fallakte übertragen von ' 
                    || coalesce(old_user.nachname, '-/-') || ', '
                    || coalesce(old_user.vorname,  '-/-') || ' an '
                    || coalesce(new_user.nachname, '-/-') || ', '
                    || coalesce(new_user.vorname,  '-/-'))
        WHERE
            id = lid;
    END IF;
    RETURN NEW;
END;
$master_ds_editor_changed$ LANGUAGE plpgsql;

CREATE TRIGGER master_ds_editor_changed_trigger AFTER
    UPDATE ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE master_ds_editor_changed();


--
-- Update master_ds_created_logbook
--
-- Since it's a trigger functions, drop the trigger first and re-create
-- it afterwards.

DROP TRIGGER master_ds_created_logbook_trigger ON master_tbl;


CREATE OR REPLACE FUNCTION master_ds_created_logbook() RETURNS trigger AS $master_ds_created_logbook$
DECLARE
    lid integer;
BEGIN
    SELECT INTO lid create_logbuch_ds(NEW.id, NULL);
    UPDATE ka_logbuch_tbl_view
        SET art = 0, typ = -1,
            notiz = notiz_from_plaintext('Fallakte angelegt.')
    WHERE
        id = lid;
    RETURN NEW;
END;
$master_ds_created_logbook$ LANGUAGE plpgsql;

CREATE TRIGGER master_ds_created_logbook_trigger AFTER
    INSERT ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE master_ds_created_logbook();


--
-- Update existing logbook entries.
--
UPDATE ka_logbuch_tbl SET notiz = notiz_from_plaintext(notiz);
