# -*- coding: utf-8 -*-
#
# Copyright 2012, 2014 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
# Björn Ricks <bjoern.ricks@intevation.de>
# Roland Geider <roland.geider@intevation.de>

"""The application's Globals object"""

import os
import logging

from pylons.controllers.util import abort

from formed.meta.io.document import openDocument

import mpulsweb.lib.app_globals
import mpulsweb.lib.helpers as h
from mpulsweb.lib.navigation import get_navigation

from jmdweb.model.struktur import is_struktur_user, get_jmd_agency
from jmdstrukturweb.lib.navigation import IQTreeBuilder


log = logging.getLogger(__name__)


class Globals(mpulsweb.lib.app_globals.Globals):

    def __init__(self, config):
        super(Globals, self).__init__(config)

        self.report_template = config.get("mpuls.app.report_template")
        if self.report_template is None:
            raise RuntimeError("mpuls.app.report_template is not configured")
        if not os.path.exists(self.report_template):
            raise RuntimeError("%r, named by mpuls.app.report_template,"
                               " does not exist" % self.report_template)
        self.odf_host = config.get("odfcast.host")
        self.odf_port = config.get("odfcast.port")

    def _load_formedtree_from_file(self, formed_file):
        # Build FormedTree. This tree will work as template for all cases
        log.debug("loading formedtree with rules for JMD-Struktur from %r",
                  formed_file)
        return openDocument(formed_file, generate_rules=True)

    def before_hook(self):
        if self.is_evaluation_server:
            pass
        elif not h.hasRole(['admin']) and not is_struktur_user():
            abort(401)

    def get_navigation(self, id, openfolders=None):
        return get_navigation(id, openfolders=openfolders,
                              builder_cls=IQTreeBuilder)

    def get_enabled_evaluations(self):
        agency = get_jmd_agency()
        is_jmd_iq = agency.get_value("jmd_iq", False)
        evaluations = self.mpuls_config.get('evaluations', 'enabled')
        return [evaluation
                for evaluation in evaluations
                if evaluation["id"] != "4" or is_jmd_iq]
