import logging
import hashlib
import re

from pylons import request, response, tmpl_context as c, url, app_globals as g
from pylons.controllers.util import redirect

from mpulsweb.lib.base import BaseCsrfController, render, session, \
    auth_cookie_name, restrict_to_POST
from mpulsweb.lib.security import checkLogin, generateID, userIdentity, \
    store_dbname_in_session, get_dbname_from_http_header
import mpulsweb.lib.helpers as h


log = logging.getLogger(__name__)


class AuthController(BaseCsrfController):

    def index(self):
        return self.login()

    def logout(self):
        session.delete()
        response.delete_cookie(auth_cookie_name())
        return render('/auth/logout.mako')

    def login(self):
        if g.mpuls_config.is_enabled('common', 'showagencychooser'):
            dbid = get_dbname_from_http_header() or ".*"
            c.filter = re.compile('%s\d{0,2}' % dbid)
        return render('/auth/login.mako')

    @restrict_to_POST
    def loginAction(self):
        try:
            login = request.params.getone('username')
            passwd = request.params.getone('password')

            session.force_new()  # start a new session

            # Try to get the name of the db to connect to from the request
            # params and check if it is contained in the list of defined
            # databases.
            dbname = request.params.get('dbname')
            if dbname:
                dblist = h.get_db_selectionlist()
                if dbname in dblist:
                    store_dbname_in_session(dbname)

            # Check login
            (user, full_login) = checkLogin(login, passwd)
            if user:
                if g.login_db:
                    session['LAST_SUCCESSFUL_LOGIN'] = \
                        g.login_db.get_success_login(full_login)
                    session.save()
                    g.login_db.store_success_login(full_login)
                c.login_failed = False
                shared = generateID()
                session['USER_AUTHORIZED'] = user
                session['AUTH'] = hashlib.md5(shared + userIdentity()).digest()
                session.save()

                # Does only work work with enabled ssl. secure means that the
                # cookie will only be sent over a secure connection.
                # response.set_cookie('mpuls_auth', shared, secure=True)
                response.set_cookie(auth_cookie_name(), shared, httponly=True)

                redirect(url("/"))
            else:
                if g.login_db and full_login is not None:
                    g.login_db.store_failed_login(full_login)
                c.login_failed = True
                if dbname:
                    c.dbname = dbname
                return self.login()

        except KeyError:
            log.error('Error while fetching username and password')
            if g.login_db:
                g.login_db.store_failed_login(login)
            session.delete()  # session must be deleted on errors
        return self.login()

    def accountDeactivated(self):
        try:
            del session['USER_AUTHORIZED']
            session.save()
        except KeyError:
            pass
        return render('/auth/account_deactivated.mako')
