# -*- coding: utf-8 -*-
# Copyright 2022 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of MPULS (CoMPUter-based case fiLeS).
#
# MPULS is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# MPULS is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with MPULS. If not, see <http://www.gnu.org/licenses/>.


import json

from pylons import app_globals as g

import formed.meta.data


class Column(object):

    """Description of a column in the case overview

    Instance variables:

    title

        The heading of the column

    fieldname

        The formedtree field name. This is used when generating the URL
        for an entry in the case overview

    style

        CSS class used in the style attribute of the HTML TH-element.

    roles

        list with the names of the roles for which the field should be
        shown.

    get_value

        Python callable that is called with the case object as argument
        and returns the formatted value for the column.
    """

    def __init__(self, fieldname, title, style, roles, get_value,
                 repeat_group=None):
        self.fieldname = fieldname
        self.title = title
        self.style = style
        self.roles = roles
        self.get_value = get_value
        self.repeat_group = repeat_group

    def __repr__(self):
        return ("%s(%r, %r, %r, %r, %r, %r)"
                % (self.__class__.__name__, self.fieldname, self.title,
                   self.style, self.roles, self.get_value, self.repeat_group))

    def preset_fieldname(self):
        suffix = "_agg" if self.repeat_group is not None else ""
        return self.fieldname + suffix

    def search_retrieve_expr(self, master_tbl):
        if self.repeat_group is not None:
            return ("ARRAY(SELECT k.%(fieldname)s"
                    "        FROM %(rg)s_tbl_view k"
                    "       WHERE k.master_id = %(master)s.id)"
                    " AS %(preset_name)s"
                    % {"fieldname": self.fieldname,
                       "preset_name": self.preset_fieldname(),
                       "rg": self.repeat_group,
                       "master": master_tbl})
        elif self.fieldname == "zugriff":
            return "st.zugriff"
        else:
            return master_tbl + "." + self.fieldname


class Overview(object):

    def __init__(self, name, columns):
        self.name = name
        self.columns = columns
