# -*- coding: utf-8 -*-
import logging
from formed.instance.tree import InstanceLeaf
from formed.meta.data import DateLeaf

from mpulsweb.lib.base import g
from mpulsweb.lib.translation import _
from mpulsweb.lib.db import db
from mpulsweb.lib.helpers import format_date, format_number 

log = logging.getLogger(__name__)

def walk_it(node, type=None):
    items = []
    items.append(node)
    if not isinstance(node, InstanceLeaf):
        for c in node.getChildren():
            items.extend(walk_it(c))
    return items

def copy_field(src_id=None, src_field=None, dst_id=None, dst_field=None):
    data = {}
    factory = g.get_instance_factory()
    sit = factory.loadInstanceTreeByIdentifier(src_id)
    dit = factory.loadInstanceTreeByIdentifier(dst_id)
    item = sit.getItem("%s:%s" % (src_field, src_id))
    if isinstance(item, DateLeaf):
        value = format_date(item.getValue()) 
    else:
        value = str(item.getValue())
    data["%s:%s" % (dst_field, dst_id)] = value
    log.debug(data)
    dit.setData(data)
    dit.commit()

def phase_time_in_weeks(ti):
    '''Used in case digest to display the time of the phases in weeks'''
    if ti is None:
        return None 
    t = float(ti.days) / 7.0 + ti.seconds / (60.0 * 24.0 * 7.0)
    if t < 1:
        return _('< 1 week')
    else:
        return format_number(t, 1)


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
