## -*- coding: utf-8 -*-
<%!  import mpulsweb.lib.helper.filters as F %>
<%!
   import mpulsweb.lib.searchexpr as S
   import mpulsweb.lib.sqlexpr as E
   from mpulsweb.lib.translation import N_

   match_operators = {
       E.EQ: N_("Equals to"),
       E.LT: N_("Less than"),
       E.LE: N_("Less or equal to"),
       E.GT: N_("Greater than"),
       E.GE: N_("Greater or equal to"),
       E.Startwith: N_("Starts with"),
       E.Substring: N_("Contains"),
       "Between": N_("Between"),
       E.IS_NULL: N_("Unspecified"),
   }
   match_operators_date = {
       E.EQ: N_("On"),
       E.LT: N_("Before"),
       E.LE: N_("Before or on"),
       E.GT: N_("After"),
       E.GE: N_("After or on"),
       "Between": N_("Between"),
       E.IS_NULL: N_("Unspecified"),
   }
%>
<link rel="stylesheet" href="${h.url('/styles/typeahead.css')}">

<script src="${h.url('/js/daterange.js')}"></script>
<script src="${h.url('/js/twitter-typeahead.bundle.js')}"></script>
<script>
$(document).ready(function() {

    /*
     * Autocompleter
     */
    var search = new Bloodhound({
        //datumTokenizer: Bloodhound.tokenizers.obj.whitespace('description'),
        datumTokenizer: function(d){
            // Naive tokenizer so that not only hits at the beginning of the
            // word are matched, see
            // * http://stackoverflow.com/questions/23448193/
            // * https://github.com/twitter/typeahead.js/issues/96
            var tokens = [];
            var stringSize = d.description.length;

            // multiple combinations for every available size
            // dog => d, o, g, do, og, dog
            for (var size = 1; size <= stringSize; size++){
                for (var i = 0; i+size <= stringSize; i++){
                    tokens.push(d.description.substr(i, size));
                }
            }
            return tokens;
        },
        queryTokenizer: Bloodhound.tokenizers.whitespace,
        limit: 10,
        local: ${c.formed_fields|n},
    });

    search.initialize();

    $('.typeahead').typeahead(null, {
        name: 'formed-search',
        source: search.ttAdapter(),
        displayKey: 'description',
    });
});
</script>
<%def name="maybeattr(attrname, cond)">
% if cond:
${attrname}
% endif
</%def>

## Render an option for a choice field
<%def name="option(value, desc, selected)">
<option value="${value}" ${maybeattr("selected", selected)}>${desc}</option>
</%def>

## Render a generic choice field
<%def name="render_search_choice(expr, selected_values, desc)">
<% key = expr.form_key() %>
% if len(desc.extras) > 5:
  <select multiple size="5" name="${key}_value">
    % for pk, value in desc.extras:
    ${option(pk, value,
             selected_values is not None and pk in selected_values)}
    % endfor
  </select>
% else:
  % for pk, value in desc.extras:
  <input name="${key}_value" type="checkbox" value="${pk}" id="${key}_value${pk}"
          ${maybeattr("checked",
          selected_values is not None and pk in selected_values)}>
  <label class="form-check-label" for="${key}_value${pk}">
    ${value}
  </label>
  <br>
  % endfor
% endif
</%def>

<%def name="render_search_radio(expr, selected_value, desc)">
<% key = expr.form_key() %>
% for pk, value in desc.extras:
<input name="${key}_value" type="radio" value="${pk}" id="${key}_value${pk}"
     ${maybeattr("checked", pk == selected_value)}
     >
<label class="form-check-label" for="${key}_value${pk}">
  ${value}
</label>
<br>
% endfor
</%def>

<%def name="render_search_bool(expr, selected_value, desc)">
<% key = expr.form_key() %>
% for pk, value in [(0, _("Set")), (-1,_("Not set"))]:
<input name="${key}_value" type="radio" value="${pk}" id="${key}_value${pk}"
     ${maybeattr("checked", pk == selected_value)}
     >
<label class="form-check-label" for="${key}_value${pk}">
  ${value}
</label>
<br>
% endfor
</%def>

<%def name="render_match_operator(op, selected, type)">
<%
   opname = op
   if not isinstance(opname, basestring):
       opname = op.__name__
%>
%if type == "date":
    ## Special case handling for date because they
    ## should be named differently.
    ${option(opname, _(match_operators_date[op]), selected)}
%else:
    ${option(opname, _(match_operators[op]), selected)}
%endif
</%def>


<%def name="render_search_entry(expr, desc)">
<%
    extra_css_classes = []
    extra_range_classes = ["input-group"]
    key = expr.form_key() 
    if desc.type == "date":
        operators = [E.EQ, E.LT, E.LE, E.GT, E.GE, "Between", E.IS_NULL]
        extra_css_classes.append("mpuls-datepicker")
        extra_range_classes.append("input-daterange")
    elif desc.type == "text":
        operators = [E.Startwith, E.Substring, E.EQ]
    elif desc.type == "int":
        operators = [E.EQ, E.LT, E.LE, E.GT, E.GE, E.IS_NULL]

    if isinstance(expr, S.Between):
        current_op = "Between"
        value_low = h.format_object(expr.low)
        value_high = h.format_object(expr.high)
        value = value_low + "-" + value_high
    else:
        current_op = expr.op
        value = h.format_object(expr.value)
        value_low = ""
        value_high = ""

    if desc.error is not None:
        value = desc.error.value
        extra_css_classes.extend(["error_box", "error"])

%>
<select name="${key}_type" style="width:30%;">
  % for op in operators:
  ${render_match_operator(op, op == current_op, desc.type)}
  % endfor
</select>
<input type="text" name="${key}_value" value="${value}"
       class="${" ".join(extra_css_classes)}" style="width:65%;">
%if "Between" in operators:
<span style="display: none;" class="${" ".join(extra_range_classes)}">
  <input type="text" name="${key}_value_low" value="${value_low}"
         class="form-control" style="width:25%">
  <span>bis</span>
  <input type="text" name="${key}_value_high" value="${value_high}"
         class="form-control" style="width:25%">
</span>
%endif
</%def>


<%def name="render_search_case_id(expr, desc)">
<%
  key = expr.form_key()
%>
<input type="text" name="${key}_value"
       % if desc.error is not None:
       value="${desc.error.value}"
       class="error_box error"
       % else:
       value="${h.format_object(expr.value)}"
       % endif
       style="width:65%;">
</%def>


<%def name="render_search_updated(expr, desc)">
<%
    key = expr.form_key()
    operators = [E.EQ, E.LT, E.LE, E.GT, E.GE, E.IS_NULL]
    current_op = expr.op
    value = h.format_object(expr.value)
%>
<select name="${key}_type" style="width:30%;">
  % for op in operators:
  ${render_match_operator(op, op == current_op, desc.type)}
  % endfor
</select>
<input type="text" name="${key}_value"
       % if desc.error is not None:
       value="${desc.error.value}"
       class="error_box error mpuls-datepicker"
       % else:
       value="${value}"
       class="mpuls-datepicker"
       % endif
       style="width:65%;">
</%def>


<%def name="render_search_responsibilities(expr, desc)">
<%
    selected = []
    if expr.own:
        selected.append(1)
    if expr.standin:
        selected.append(2)
    if expr.changed_editor:
        selected.append(3)
%>
% if not h.hasRole(['cm']):
<p>
  ${_("Note: This criterion matches cases for which the searching user is editor or standin and thus is only useful for case-management users.")}
</p>
% endif
${render_search_choice(expr, selected, desc)}
</%def>


<%def name="render_search_needle(expr, desc)">
<% key = expr.form_key() %>
<input type="text" name="${key}_value" value="${expr.pattern}"
       style="width:90%;">
</%def>


<%def name="render_search_input(expr, desc)">
% if isinstance(expr, (S.Choice, S.Status)):
  ${render_search_choice(expr, expr.values, desc)}
% elif isinstance(expr, S.Radio):
  ${render_search_radio(expr, expr.value, desc)}
% elif isinstance(expr, S.Bool):
  ${render_search_bool(expr, expr.value, desc)}
% elif isinstance(expr, (S.Field, S.Between)):
  ${render_search_entry(expr, desc)}
% elif isinstance(expr, S.Responsibilities):
  ${render_search_responsibilities(expr, desc)}
% elif isinstance(expr, S.Needle):
  ${render_search_needle(expr, desc)}
% elif isinstance(expr, S.Updated):
  ${render_search_updated(expr, desc)}
% endif
</%def>


<%def name="display_search_input(expr, desc)">
<%
  if isinstance(expr, (S.Choice, S.Status)):
      return "unfolded" if expr.values is not None else "folded"
  elif isinstance(expr, (S.Radio, S.Bool)):
      return "unfolded" if expr.value is not None else "folded"
  elif isinstance(expr, S.Field):
      return "unfolded" if expr.value is not None and expr.value != "" else "folded"
  elif isinstance(expr, S.Between):
      return "unfolded" if (expr.low is not None and expr.low != "") or (expr.high is not None and expr.high != "") else "folded"
  elif isinstance(expr, S.Responsibilities):
      return "unfolded" if expr.own or expr.standin or expr.changed_editor else "folded"
  elif isinstance(expr, S.Needle):
      return "unfolded" if expr.pattern > "" else "folded"
  elif isinstance(expr, S.Updated):
      return "unfolded" if expr.value != None else "folded"
%>
</%def>


<%def name="search_expr(expr, desc)">
<% key = expr.form_key() %>
<% folding = display_search_input(expr, desc) %>
<div class="w-50">
  <div class="row">
    <div class="col-9" id="heading${key}">
      <label>
        <a class="${folding} info-item " id="label_${key}" onClick="toggle_folding('${key}')">
          ${desc.description}
        </a>
      </label>
    </div>
    <div class="col">
      <button type="submit" alt="${_('Delete')}" name="delete_field_${key}" id="delete_field_${key}"
              title="${_('Delete')}" class="btn btn-sm icon-btn">
        <svg class="svg-icon svg-warning" aria-labelledby="title" role="img">
          <title id="title">${_('Delete')}</title>
          <use xlink:href="#delete" />
        </svg>
      </button>
    </div>
  </div>
  <div class="row">
    <% folding_display = "none" if folding == "folded" else "block" %>
    <div id="${key}" style="display: ${folding_display}">
      <div class="panel-body mb-2">
        % if desc.error is not None:
        <div class="error">
          ${desc.error}
        </div>
        % endif

        ${self.render_search_input(expr, desc)}
      </div>
    </div>
    <input id="folded.${key}" type="hidden" value="none">
  </div>
</div>
</%def>

% if g.mpuls_config.is_enabled('case-module', 'search'):
<form method="post" name="case_search" accept-charset="UTF-8"
      action="${h.url(controller='case_overview', action='search')}">
  ${ h.csrf_token() }
  <button onClick="javascript:return false;" name="default-submit" style="display: none;"></button>

<div class="row mb-3 d-print-none">
  <div class="col-8" id="current_case_search">
    <h4 class="text-muted">${_('Your current search parameters')}</h4>
    <div class=" d-flex flex-wrap">
      % for expr, extra in c.search_exprs:
        ${search_expr(expr, extra)}
      % endfor
    </div>
  </div>

    <div class="col">
        <h4 class="text-muted">Suchparameter hinzufügen</h4>
        <div class="row">
            <div class="col">
                <input type="text" name="new_field_description" id="new_field_description" maxlength="256"
                    class="form-control typeahead" placeholder="Neuer Suchparameter"
                    style="width:100% !important">
                % if c.add_field_error:
                    <small class="alert-danger errortipp">${c.add_field_error}</small>
                % endif
            </div>
            <div class="col" style="padding-left: 0;">
                <button alt="${_('Add')}" name="add_field" id="add_field"
                    title="${_('Add')}" class="btn btn-sm icon-btn" id="param-btn">
                    <svg class="svg-icon" aria-labelledby="title" role="img">
                      <title id="title">${_('Add')}</title>
                      <use xlink:href="#plus-circle-solid" />
                    </svg>
                </button>
                <a tabindex="0" data-toggle="popover" data-trigger="focus" data-html="true"
                   data-placement="top" title="${_('Help')}"
                   data-content="Ausführliche Informationen finden Sie auf der <a href='${h.url(controller='help', action='help_flexible_search')}' target='_blank'>Hilfeseite für die flexible Suche</a>">
                      <svg class="svg-icon" aria-labelledby="title" role="img">
                        <title id="title">${_('Help')}</title>
                        <use xlink:href="#help" />
                      </svg>
                  </a>
            </div>
            <!-- Bind Enter keypress to the right submitbutton for new field description -->
            <script>
              // Get the input field
              var new_field_description = document.getElementById("new_field_description");

              // Execute a function when the user press a key on the keyboard
              new_field_description.addEventListener("keydown", function(event) {
                // Number 13 is the "Enter" key on the keyboard
                if (event.keyCode == 13) {
                    // Cancel the default action, if needed
                    event.preventDefault();
                    // Trigger the button element with a click
                    if (new_field_description.value != '') {
                        document.getElementById("add_field").click();
                    }
                }
              });
            </script>
        </div>

        <h4 class="text-muted">${_('Saved searches')}</h4>
        <ul class="pl-0">
            % for num, f in enumerate(c.saved_searches):
                <!-- TODO: class="saved-search-item" doesn't seem to be defined any more -->
                <li class="saved-search-item mb-1"><a class="info-item" href="${h.url(controller='savedsearch', action='load', id=f.id)}"
                >${ F.shorten(f.get_name(),60) }</a></li>
            % endfor
        </ul>
        <a href="${h.url(controller='savedsearch', action='overview')}" class="btn btn-outline-primary btn-sm"
            id="manageSearch">${_('Manage saved searches')}</a>
    </div>
</div>

    <div class="form-group d-print-none">
        <input type="submit" value="${_('Perform search')}" name="perform_search"
           class="btn btn-primary" id="perform_search">
        <input type="submit" value="${_('Save current search')}" name="to_json"
           class="btn btn-outline-primary">
        <a href="${h.url(controller='case_overview', action='reset')}"
            class="btn btn-outline-primary">${_('Reset')}</a>
    </div>

    <!-- Bind Enter keypress to the right submitbutton for all current search description -->
    <script>
      // Get all fields of the current search
      var current_case_search = document.getElementById("current_case_search");

      // Execute a function when the user press a key on the keyboard
      current_case_search.addEventListener("keydown", function(event) {
        // Number 13 is the "Enter" key on the keyboard
        // Delete buttons should be able to continue to be executed with "Enter".
        if (event.keyCode == 13 && event.target.tagName !== "BUTTON" ) {
            // Cancel the default action, if needed
            event.preventDefault();
            // Trigger the button element with a click
            document.getElementById("perform_search").click();
        }
      });
    </script>


</form>
% endif

<%def name="showCount(num)">
  ${"(%s)" % num}
</%def>
<%def name="buildFormHeader()">
  <form method="post" name="case_search" action="${h.url(controller='case_overview', action='search')}" accept-charset="UTF-8">
      ${ h.csrf_token() }
</%def>
<%def name="buildSearchReset()">
  <a href="${h.url('#')}">${_('Reset search')}</a>
</%def>
<%def name="buildSearchExtend()">
  % if c.extended_search:
    <a href="${h.url('#')}">${_('Hide extended search')}</a>
  % else:
    <a href="${h.url('#')}">${_('Show extended search')}</a>
  % endif
  ${h.getHelp('extended_search')}
</%def>
