SET ROLE :adm_ka_owner;
BEGIN;

CREATE TABLE tags (
    id          SERIAL PRIMARY KEY,
    label       VARCHAR(256) NOT NULL,
    description TEXT,
    systemtag   BOOLEAN DEFAULT TRUE
);

CREATE TABLE nm_tags_cases (
    id SERIAL PRIMARY KEY,
    master_id   integer NOT NULL REFERENCES master_tbl(id) ON DELETE CASCADE,
    tag_id    integer NOT NULL REFERENCES tags(id)       ON DELETE CASCADE,
    created   TIMESTAMP DEFAULT now(),
    UNIQUE (master_id, tag_id)
);

-- Insert application specific tag entries!

-- Move tags serial: userdefined tags start at 1000
ALTER SEQUENCE tags_id_seq RESTART 10000;

CREATE OR REPLACE FUNCTION delete_tag_ds ( tagid integer) RETURNS VOID
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM tags WHERE id = tagid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_tag_ds(integer)
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_tag_ds(integer)
                TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION create_tag_ds () RETURNS INTEGER
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
     INSERT INTO tags (label, description)
         VALUES ('', NULL);
     RETURN currval('tags_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_tag_ds()
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_tag_ds()
                TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION create_nm_tag_case_ds ( tagid integer,
    masterid integer ) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    INSERT INTO nm_tags_cases (tag_id, master_id)
        VALUES (tagid, masterid);
    RETURN currval('nm_tags_cases_id_seq');
END;
$$ ;

REVOKE ALL ON FUNCTION create_nm_tag_case_ds(integer, integer)
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_nm_tag_case_ds(integer, integer)
                TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION create_nm_tag_case_ds(integer, integer)
                TO GROUP :admin_group;

CREATE OR REPLACE FUNCTION delete_nm_tag_case_ds ( tagid integer,
    masterid integer ) RETURNS VOID
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
    DELETE FROM nm_tags_cases
            WHERE master_id = masterid AND tag_id = tagid;
END;
$$ ;

REVOKE ALL ON FUNCTION delete_nm_tag_case_ds(integer, integer)
                FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_nm_tag_case_ds(integer, integer)
                TO GROUP :cm_group;
GRANT EXECUTE ON FUNCTION delete_nm_tag_case_ds(integer, integer)
                TO GROUP :admin_group;

-- CREATE OR REPLACE FUNCTION mpuls_tag() RETURNS TRIGGER AS $$
-- BEGIN
--     IF TG_OP = 'DELETE' THEN
--         RETURN OLD;
--     ELSE
--     	PERFORM check_case(NEW.id);
--         RETURN NEW;
--     END IF;
-- END;
-- $$ LANGUAGE plpgsql;

-- SET ROLE postgres;
-- 
-- CREATE OR REPLACE FUNCTION check_case(ds_id integer) RETURNS void 
-- AS $$
-- import socket
-- 
-- UNIX_SOCKET = "/tmp/tagd-socket"
-- 
-- db = plpy.execute("SELECT current_database() as db")[0]["db"]
-- 
-- s = None
-- try:
--     try:
--         s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
--         s.connect(UNIX_SOCKET)
--         s.send("%s %d\n" % (db, ds_id))
--     except:
--         plpy.warning('Can not connect tagd socket')
-- finally:
--     if s:
--         try: s.close()
--         except: pass
--           
-- $$ LANGUAGE plpythonu;


COMMIT;
\i sql/install/views/09_tagging.sql

-- Example:
--

--  DELETE FROM tags;
--  lINSERT INTO tags (id, label, description) VALUES (0, 'Alle', 'Jede Fallakte hat einen Tag');
--  lINSERT INTO tags (id, label, description) VALUES (1, 'Grundsätzlich auswertbar', 'Die Fallakte ist konsistent und weist die benötigten Validitäten auf, die für eine grundsätzliche Auswertung erfüllt sein müssen. Die Fallakte steht in Auswertungen zur Verfügung');
--  lINSERT INTO tags (id, label, description) VALUES (2, 'Berichtsfähig', 'Eine Fallakte ist grundsätzlich auswertbar. Sie befindet sich darüber hinaus mindestens im laufendem CM und enthält eine Geschlechtsangabe. Die Fallakte steht für Auswertungen wie den Verwendungsnachweis zur Verfügung');
--  lINSERT INTO tags (id, label, description) VALUES (10, 'Datumsangaben konsistent', 'Alle Datumsangaben in der Fallakte befinden sich in chronologisch korrekter Reihenfolge und liegen nicht in der Zukunft');
--  lINSERT INTO tags (id, label, description) VALUES (11, 'Phase ist gesetzt', 'Die Phase der Fallakte ist nicht unbekannt');
--  lINSERT INTO tags (id, label, description) VALUES (12, 'Synchronität der Phasen ist gegeben', 'Die in der Fallakte angegebenen Datumsangaben sind nicht weiter ausgefüllt, als es die aktuell gesetzte Phase. Z.b ein gesetzten Beendigungsdatum bei laufender Phase');
--  lINSERT INTO tags (id, label, description) VALUES (13, 'Junger Mensch ist in der Einzelfallhilfe', 'Die Betreuung des jungern Menschen findet im Rahmen der
--  l    Einzelfallhilfe statt. Für den jungen Menschen wurde im
--  l    Phasenmodell mindestens ein laufendes Case Management bzw. Case Work
--  l    dokumentiert.');
--  lINSERT INTO tags (id, label, description) VALUES (14, 'Geschlechtsangabe angegeben', 'Die Geschlechtsangabe des jungen Menschen ist angegeben und hat nicht den Wert "unbekannt"');
--  lINSERT INTO tags (id, label, description) VALUES (15, 'Formale Angabe "Entscheidung zur Einzelfallhilfe" in der Fallakte sind passend zur gesetzten Phase', 'Die Regeln, die sicherstellen das die Phase Einzelfallhilfe bzw. Beratung nur dann gestartet werden können wenn eine entsprechende Begleitung bzw. Notwendigkeit zur Übernahme dokumentiert wurde, sind erfüllt.');

