#!/bin/sh

usage()
{
cat << EOF
usage: $0 options

This script will copy a given exportfile to the exporthost using scp.

OPTIONS:
   -h      Show this message
   -d      Date (YYYY-MM-DD) of the export to copy. Default: Today
   -l      Local name of the export.
   -r      Remote name of the export. Defaults to local name.
   -s      Server where the data will be copied.
   -v      Verbose (unused)
EOF
}

# DEFAULTS
VERBOSE=0
DATE=`date +"%Y-%m-%d"`
TYPE=
RTYPE=
EXPORTHOST=

# Parse Options
while getopts "hvr:l:f:d:s:" OPTION
do
        case $OPTION in
            h)
                usage
                exit 1
                ;;
            d)
                DATE=$OPTARG
                ;;
            l)
                TYPE=$OPTARG
                ;;
            r)
                RTYPE=$OPTARG
                ;;
            s)
                EXPORTHOST=$OPTARG
                ;;
            v)
                VERBOSE=1
                ;;
            ?)
                # Will be set if the optstring begins with a ":". In this case
                # all options not defined in the optstring will OPTION set to
                # "?". Otherwise the shell reports an error on the unknown
                # option.
                usage
                exit
                ;;
        esac
done

# Check Options
if [[ -z $TYPE ]]
then
    echo "Error! Name for export required!";
    usage;
    exit 1;
fi
if [[ -z $RTYPE ]]
then
    RTYPE=${TYPE}
fi

if [[ -z $EXPORTHOST ]]
then
    echo "Error! Export Host is required!";
    usage;
    exit 1;
fi

# Finally excecute program

USER=mpuls

BASEDIR=/home/${USER}/${TYPE}/releases/current/evaluation-transfer
DATADIR=${BASEDIR}/data

REMOTEBASEDIR=/home/${USER}/${RTYPE}/releases/current/evaluation-transfer
REMOTEDATADIR=${REMOTEBASEDIR}/data

EXPORTFILE=${DATADIR}/${TYPE}-${DATE}.db.bz2
REMOTEFILE=${REMOTEDATADIR}/${RTYPE}-${DATE}.db.bz2

scp -qp -o "StrictHostKeyChecking=no" \
    -o "UserKnownHostsFile=/dev/null" \
    ${EXPORTFILE} ${USER}@${EXPORTHOST}:${REMOTEFILE}

rm ${EXPORTFILE}

exit 1
