# -*- coding: utf-8 -*-
#
# Copyright (c) 2012, 2013 Intevation GmbH
#
# libformed is free software under the terms of the
# GNU General Public License Version 3. See file
# LICENSE coming with the source of FormEd for details.
#
# Author:
# Roland Geider  <roland.geider@intevation.de>

import codecs
from time import gmtime, strftime

from formed.export.sql import EVALUATION_VIEW, ExportAsSQL, SchemaCreator, \
     sqlName


DROP_VIEW = "DROP VIEW IF EXISTS %s CASCADE;\n"


class ExportDelViews(ExportAsSQL):

    @staticmethod
    def getDescription():
        return "Export document as DROP VIEW statements"

    def doExport(self, path):
        root = self.root

        creator = SchemaCreator(root)
        creator.createSchema()
        tables = creator.topological()
        arr = self.printRepeats(tables)

        out = codecs.open(path, "wb", "UTF-8")
        try:
            pfeil = out.write
            pfeil(u'--\n-- Schema generated by libformed\n-- %s\n'
                  % strftime("%a, %Y-%m-%d %H:%M:%S +0000", gmtime()))
            pfeil('-- # Revision: $Revision: fa78b7860483 $\n--\n')

            # drop main evaluation view
            pfeil('\n')
            pfeil(DROP_VIEW % EVALUATION_VIEW)

            # drop views for master table and repeat groups
            pfeil("--\n--\n-- views for master table and repeat groups\n--\n")
            for raw_table, joins in reversed(arr):
                table = raw_table.name
                pfeil(DROP_VIEW % (sqlName(table) + '_anonym_view'))
                pfeil(DROP_VIEW % (sqlName(table) + "_view"))
        finally:
            out.close()
