"""Basic support for navigation trees"""

class TreeItem(object):

    def __init__(self, name, description, children=None, parent=None):
        self.is_open = False
        self.name = name
        self.description = description
        self.parent = parent
        self.children = children
        self.key = None
        self.next = None
        self.repeat = False

    def is_child(self):
        return self.children is None

    def deepest_item(self):
        cur = self
        if cur.repeat:
            return cur
        while cur.children:
            cur = cur.children[0]
            if cur.repeat:
                break
        return cur

    def get_depth(self):
        depth = 0
        current = self
        while current.parent:
            current = current.parent
            depth += 1
        return depth

    def find_page(self):
        cur = self
        while cur:
            if cur.next:
                return cur
            cur = cur.parent
        return None

    def unfold(self):
        opened = []
        cur = self
        while cur:
            cur.is_open = True
            opened.append(cur)
            cur = cur.parent
        return opened

    def render(self, page, out, get_image, error_lookup, num=0):
        pass

    def set_open(self, flag):
        self.is_open = flag

    def build_error_lookup(self, lookup, error_pages):
        mark = self.name in error_pages
        if self.children:
            for child in self.children:
                if child.build_error_lookup(lookup, error_pages):
                    mark = True
        if mark:
            lookup.add(self)
        return mark

    def nextKey(self):
        page = self.find_page()
        if not page:
            return None
        return page.next
